==> bioinfo_tools-0.0.4.tar.gz: bioinfo_tools-0.0.4/bioinfo_tools/genomic_features/chromosome.py <==
from typing import Dict

from Bio.SeqFeature import FeatureLocation

from bioinfo_tools.genomic_features.gene import Gene


class Chromosome(object):
    def __init__(self, chromosome_id):
        self.chromosome_id = chromosome_id
        self.length = 0
        self._genes = {}
    
    def __repr__(self):
        return u"%s (%s genes)" % (self.chromosome_id, len(self.genes))
        
    def add_gene(self, gff_feature:Dict):
        gene_id = None
        
        if 'gene_id' in gff_feature:
            gene_id = gff_feature['gene_id']
        elif 'attributes' in gff_feature:
            if 'Name' in gff_feature['attributes']:
                gene_id = gff_feature['attributes']['Name']
            elif 'ID' in gff_feature['attributes']:
                gene_id = gff_feature['attributes']['ID']
        
        if not gene_id:
            raise Exception("gene_id not found in given GFF feature")
        
        gene = Gene(
            gene_id = gene_id,
            chromosome = self,
            start = gff_feature.get('start', None),
            end = gff_feature.get('end', None),
            strand = gff_feature.get('strand', None),
            **gff_feature.get('attributes', {})
        )
        for transcript in gff_feature.get('mRNA', []):
            gene.add_transcript(transcript)
        
        self._genes[gene['gene_id']] = gene
    
    @property
    def genes(self):
        return list(self._genes.values())
    
    @property
    def sorted_genes(self):
        if not hasattr(self, "_sorted_genes"):
            self._sorted_genes = sorted(self._genes.values(), key = lambda gene: gene.location.start)
        return self._sorted_genes
    
    def get_gene(self, gene_id):
        return self._genes.get(gene_id, None)

    def attach_nucleic_sequence(self, sequence):
        self.nucleic_sequence = sequence
        self.length = len(sequence)
    
    def extract_sequence(self, start, end):
        location = FeatureLocation(start, end)
        return location.extract(self.nucleic_sequence)

==> bioinfo_tools-0.0.8.tar.gz: bioinfo_tools-0.0.8/bioinfo_tools/genomic_features/chromosome.py <==
from typing import Dict, List

from Bio.SeqFeature import FeatureLocation

from bioinfo_tools.genomic_features.gene import Gene


class Chromosome(object):
    def __init__(self, chromosome_id):
        self.chromosome_id = chromosome_id
        self.length = 0
        self._genes = {}
    
    def __repr__(self):
        return u"%s (%s genes)" % (self.chromosome_id, len(self.genes))
    
    def add_gene(self, gff_feature:Dict):
        gene_id = None
        
        if 'gene_id' in gff_feature:
            gene_id = gff_feature['gene_id']
        elif 'attributes' in gff_feature:
            if 'Name' in gff_feature['attributes']:
                gene_id = gff_feature['attributes']['Name']
            elif 'ID' in gff_feature['attributes']:
                gene_id = gff_feature['attributes']['ID']
        
        if not gene_id:
            raise Exception("gene_id not found in given GFF feature")
        
        gene = Gene(
            gene_id = gene_id,
            chromosome = self,
            start = gff_feature.get('start', 0),
            end = gff_feature.get('end', 0),
            strand = gff_feature.get('strand', None),
            **gff_feature.get('attributes', {})
        )
        for transcript in gff_feature.get('mRNA', []):
            gene.add_transcript(transcript)
        
        self._genes[gene['gene_id']] = gene
        return gene
    
    @property
    def genes(self) -> List[Gene]:
        return list(self._genes.values())
    
    @property
    def sorted_genes(self) -> List[Gene]:
        if not hasattr(self, "_sorted_genes"):
            self._sorted_genes = sorted(self._genes.values(), key = lambda gene: gene.location.start)
        return self._sorted_genes
    
    def get_gene(self, gene_id) -> Gene:
        return self._genes.get(gene_id, None)

    def attach_nucleic_sequence(self, sequence):
        self.nucleic_sequence = sequence
        self.length = len(sequence)
    
    def extract_sequence(self, start, end):
        location = FeatureLocation(start, end)
        return location.extract(self.nucleic_sequence)
